@@ -1,3 +1,4 @@
+#ifdef ENABLE_DNSSD
 #ifdef ENABLE_BONJOUR
     #include "dns_sd.h"
 
@@ -252,7 +253,8 @@ static int32_t f_svc_check_resolved(t_service_context *svc_context,
     int32_t err = X502_ERR_OK;
     const char *devname_ptr = f_get_txt_rec(txt_record, "devname", &len);
     const char *serial_ptr;
-    if ((devname_ptr != NULL) && !strncmp(devname_ptr, E502_DEVICE_NAME, len)) {
+    if ((devname_ptr != NULL) &&
+        (!strncmp(devname_ptr, E502_DEVICE_NAME, len) || !strncmp(devname_ptr, E16_DEVICE_NAME, len))) {
         memcpy(svc_context->rec->devname, devname_ptr, len);
         serial_ptr = f_get_txt_rec(txt_record, "serial", &len);
         if (serial_ptr != NULL) {
@@ -277,7 +279,8 @@ static void f_add_new_svc(t_e502_eth_svc_browse_hnd browse_context, t_svc_iface_
         memset(svc_context, 0, sizeof(t_service_context));
         svc_context->rec = f_service_record_create();
         if (svc_context->rec != NULL) {
-            strncpy(svc_context->rec->service_name, svc_name, X502_INSTANCE_NAME_SIZE);
+            strncpy(svc_context->rec->service_name, svc_name, X502_INSTANCE_NAME_SIZE-1);
+            svc_context->rec->service_name[X502_INSTANCE_NAME_SIZE-1] = '\0';
             svc_context->rec->domain = malloc(domain_len);
             if (svc_context->rec->domain != NULL) {
                 strcpy(svc_context->rec->domain, domain);
@@ -526,7 +529,7 @@ static void DNSSD_API f_browse_replay (DNSServiceRef sdRef, DNSServiceFlags flag
                 rec->op_in_progr = 0;
                 break;
             case AVAHI_RESOLVER_FOUND: {
-                    size_t hostlen = strlen(host_name);
+                    size_t hostlen = strlen(host_name) + 1;
                     rec->port = port;
                     if (rec->hosttarget != NULL)
                         free(rec->hosttarget);
@@ -677,6 +680,13 @@ X502_EXPORT(int32_t) E502_EthSvcRecordGetDevSerial(t_e502_eth_svc_record_hnd rec
     return err;
 }
 
+X502_EXPORT(int32_t) E502_EthSvcRecordGetDevName(t_e502_eth_svc_record_hnd rec, char *devname) {
+    int32_t err = devname == NULL ? X502_ERR_INVALID_POINTER : E502_CHECK_SVC_REC(rec);
+    if (err == X502_ERR_OK) {
+        strcpy(devname, rec->devname);
+    }
+    return err;
+}
 
 #ifdef ENABLE_BONJOUR
 static int32_t f_wait_result(DNSServiceRef ref, t_e502_eth_svc_record_hnd rec, t_ltimer *ptmr) {
@@ -930,7 +940,7 @@ X502_EXPORT(int32_t) E502_EthSvcBrowseStop(t_e502_eth_svc_browse_hnd context) {
 X502_EXPORT(int32_t) E502_MakeDevRecordByEthSvc(t_x502_devrec *devrec, t_e502_eth_svc_record_hnd svc,
                                                 uint32_t flags, uint32_t tout) {
 
-    int32_t err = e502_make_tcp_rec(devrec, flags, tout);
+    int32_t err = e502_make_tcp_rec(devrec, flags, tout, svc->devname);
     if (err==X502_ERR_OK) {
         t_tcp_devinfo_data *devinfo_data = (t_tcp_devinfo_data *)devrec->internal->iface_data;
         devinfo_data->svc_rec = f_service_record_create_copy(svc);
@@ -951,3 +961,99 @@ int32_t e502_svc_fill_devinfo(t_tcp_devinfo_data *data) {
     }
     return err;
 }
+
+#else
+#include "e502api.h"
+X502_EXPORT(int32_t) E502_EthSvcRecordFree(t_e502_eth_svc_record_hnd rec) {
+    return X502_ERR_NOT_IMPLEMENTED;
+}
+
+X502_EXPORT(int32_t) E502_EthSvcRecordGetInstanceName(t_e502_eth_svc_record_hnd rec, char *name) {
+    return X502_ERR_NOT_IMPLEMENTED;
+}
+
+X502_EXPORT(int32_t) E502_EthSvcRecordGetDevSerial(t_e502_eth_svc_record_hnd rec, char *serial) {
+    return X502_ERR_NOT_IMPLEMENTED;
+}
+
+X502_EXPORT(int32_t) E502_EthSvcRecordResolveIPv4Addr(t_e502_eth_svc_record_hnd rec,
+                                                      uint32_t *addr, uint32_t tout) {
+    return X502_ERR_NOT_IMPLEMENTED;
+}
+
+X502_EXPORT(int32_t) E502_EthSvcRecordIsSameInstance(t_e502_eth_svc_record_hnd svc1,
+                                                     t_e502_eth_svc_record_hnd svc2) {
+    return X502_ERR_NOT_IMPLEMENTED;
+}
+
+
+X502_EXPORT(int32_t) E502_EthSvcBrowseStart(t_e502_eth_svc_browse_hnd *pcontext, uint32_t flags) {
+    return X502_ERR_NOT_IMPLEMENTED;
+}
+
+X502_EXPORT(int32_t) E502_EthSvcBrowseGetEvent(t_e502_eth_svc_browse_hnd context,
+                                                 t_e502_eth_svc_record_hnd *svc, uint32_t *pevent,
+                                               uint32_t *flags, uint32_t tout) {
+    return X502_ERR_NOT_IMPLEMENTED;
+}
+
+X502_EXPORT(int32_t) E502_EthSvcBrowseStop(t_e502_eth_svc_browse_hnd context) {
+    return X502_ERR_NOT_IMPLEMENTED;
+}
+
+X502_EXPORT(int32_t) E502_MakeDevRecordByEthSvc(t_x502_devrec *devrec, t_e502_eth_svc_record_hnd svc,
+                                                uint32_t flags, uint32_t tout) {
+    return X502_ERR_NOT_IMPLEMENTED;
+}
+#endif
+
+X502_EXPORT(int32_t) E502_SearchEthForDevicesIPv4Addr(t_x502_devrec* rec_list, uint32_t flags, uint32_t size, 
+                                        uint32_t *devcount, uint32_t event_tout, uint32_t tcp_tout){
+    
+    uint32_t count = 0;
+    int32_t err = X502_ERR_OK;
+
+    t_e502_eth_svc_browse_hnd browse_hnd;
+    err = E502_EthSvcBrowseStart(&browse_hnd, 0);
+
+    if (!err){
+        int run = 1;
+        while (run) {
+            t_e502_eth_svc_record_hnd svc_hnd;
+            uint32_t event;
+            // обязательно вернет значение после tout мс
+            err = E502_EthSvcBrowseGetEvent(browse_hnd, &svc_hnd, &event, &flags, event_tout);
+            if (!err){
+                if (E502_ETH_SVC_EVENT_NONE == event) {
+                    run = 0;
+                } else {
+                    if ((NULL != rec_list) && (count < size)) {
+                        int32_t res = E502_MakeDevRecordByEthSvc(&rec_list[count], svc_hnd, flags, tcp_tout);
+                        if (X502_ERR_OK != res){
+                            break;
+                        }
+                        uint32_t ip_addr;
+                        E502_EthSvcRecordResolveIPv4Addr(svc_hnd, &ip_addr ,tcp_tout);
+
+                        sprintf(rec_list[count].location, "%d.%d.%d.%d",
+                            (ip_addr>>24) & 0xFF,
+                            (ip_addr>>16) & 0xFF,
+                            (ip_addr>>8) & 0xFF,
+                            (ip_addr>>0) & 0xFF);
+                        rec_list[count].location_type = X502_LOCATION_TYPE_ADDR;
+                    }
+                    count++;
+                }
+            } else {
+                run = 0;
+            }
+        }
+    }
+    err = E502_EthSvcBrowseStop(browse_hnd);
+
+    if (NULL != devcount){
+        *devcount = count;
+    }
+    
+    return err != X502_ERR_OK ? err  : count > (uint32_t)size ? (int32_t)size : (int32_t)count ;
+}
