@@ -1,3 +1,4 @@
+#ifdef ENABLE_TCP
 #include "e502api_private.h"
 #include "e502_tcp_protocol.h"
 #include "ltimer.h"
@@ -35,8 +36,6 @@
     //#include <stropts.h>
 
     #include <linux/sockios.h>
-    #include <asm-generic/ioctls.h>
-
 
 
     typedef int t_socket;
@@ -60,15 +59,20 @@
 #define E502_TCP_REQ_TOUT               5000
 #define E502_TCP_STOP_WAIT_TOUT         5000
 #define X502_MUTEX_TCP_IOCTL_LOCK_TOUT  5000
+#define X502_MUTEX_TCP_DATA_LOCK_TOUT   5000
 
 
 
 #define TCP_CTL_REQ_MAX_SIZE   512
 #define TCP_IN_STREAM_BUF_MIN  128
 
+#define TCP_IOCTL_INLINE_MAX_DATA_SIZE  64
 
-
-
+#if 0
+    #define dprintf(...)                    fprintf(stderr, __VA_ARGS__)
+#else
+    #define dprintf(...)
+#endif
 
 typedef struct {
     t_socket cmd_sock;
@@ -77,6 +81,8 @@ typedef struct {
     uint32_t open_tout;
     uint16_t data_port;
     t_mutex  ioctl_mutex;
+    t_mutex  data_mutex;
+    uint32_t data_chs_en;
 
     uint32_t recv_part_wrd; /**< принятое неполностью слово */
     uint32_t send_part_wrd; /**< переданное неполностью слово */
@@ -356,7 +362,10 @@ static int32_t f_send_exact(t_socket s, const uint8_t *buf, uint32_t size, t_lti
 static int32_t f_iface_gen_ioctl(t_x502_hnd hnd, uint32_t cmd_code, uint32_t param,
                                  const void* snd_data, uint32_t snd_size,
                                  void* rcv_data, uint32_t recv_size, uint32_t* recvd_size, uint32_t tout) {
-    t_e502_tcp_cmd_hdr cmd_hdr;
+    struct {
+        t_e502_tcp_cmd_hdr hdr;
+        uint8_t data[TCP_IOCTL_INLINE_MAX_DATA_SIZE];
+    } cmd;
     t_e502_tcp_resp_hdr cmd_resp;
     int32_t err = X502_ERR_OK;
     t_ltimer tmr;
@@ -366,16 +375,26 @@ static int32_t f_iface_gen_ioctl(t_x502_hnd hnd, uint32_t cmd_code, uint32_t par
     err = osspec_mutex_lock(iface_data->ioctl_mutex, X502_MUTEX_TCP_IOCTL_LOCK_TOUT);
     if (err == X502_ERR_OK) {
         ltimer_set(&tmr, LTIMER_MS_TO_CLOCK_TICKS(tout == 0 ? E502_TCP_REQ_TOUT : tout));
-        cmd_hdr.sign = E502_TCP_CMD_SIGNATURE;
-        cmd_hdr.cmd = cmd_code;
-        cmd_hdr.par = param;
-        cmd_hdr.data_len = snd_size;
-        cmd_hdr.resp_len = recv_size;
+        cmd.hdr.sign = E502_TCP_CMD_SIGNATURE;
+        cmd.hdr.cmd = cmd_code;
+        cmd.hdr.par = param;
+        cmd.hdr.data_len = snd_size;
+        cmd.hdr.resp_len = recv_size;
 
-        err = f_send_exact(s, (uint8_t*)&cmd_hdr, E502_TCP_CMD_HDR_SIZE, &tmr);
-        if ((err == X502_ERR_OK) && (snd_size > 0)) {
-            err = f_send_exact(s, snd_data, snd_size, &tmr);
+        /* чтобы избежать двух передач по TCP, если данных, передаваемых с командой,
+         * меньше заданного порога, то объединяем их вместе с заголовком и посылаем
+         * за один вызов send */
+        if (snd_size <= TCP_IOCTL_INLINE_MAX_DATA_SIZE) {
+            if (snd_size > 0)
+                memcpy(cmd.data, snd_data, snd_size);
+            err = f_send_exact(s, (uint8_t*)&cmd, E502_TCP_CMD_HDR_SIZE + snd_size, &tmr);
+        } else {
+            err = f_send_exact(s, (uint8_t*)&cmd.hdr, E502_TCP_CMD_HDR_SIZE, &tmr);
+            if (err == X502_ERR_OK) {
+                err = f_send_exact(s, snd_data, snd_size, &tmr);
+            }
         }
+
         if (err == X502_ERR_OK) {
             err = f_recv_exact(s, (uint8_t*)&cmd_resp, E502_TCP_CMD_RESP_SIZE, &tmr);
             if (err == X502_ERR_RECV_INSUFFICIENT_WORDS) {
@@ -450,7 +469,10 @@ static int32_t f_iface_open(t_x502_hnd hnd, const t_x502_devrec *devrec) {
             iface_data->open_tout = devinfo_data->open_tout;
             iface_data->data_port = devinfo_data->data_port;
             iface_data->ioctl_mutex = osspec_mutex_create();
-            if (iface_data->ioctl_mutex == OSSPEC_INVALID_MUTEX) {
+            iface_data->data_mutex = osspec_mutex_create();
+            iface_data->data_chs_en = 0;
+            if ((iface_data->ioctl_mutex == OSSPEC_INVALID_MUTEX)
+                    || (iface_data->data_mutex == OSSPEC_INVALID_MUTEX)) {
                 err = X502_ERR_MUTEX_CREATE;
             } else {
                 hnd->iface_data = iface_data;
@@ -471,6 +493,8 @@ static int32_t f_iface_open(t_x502_hnd hnd, const t_x502_devrec *devrec) {
             if (err != X502_ERR_OK) {
                 if (iface_data->ioctl_mutex != OSSPEC_INVALID_MUTEX)
                     osspec_mutex_destroy(iface_data->ioctl_mutex);
+                if (iface_data->data_mutex != OSSPEC_INVALID_MUTEX)
+                    osspec_mutex_destroy(iface_data->data_mutex);
                 hnd->iface_data = NULL;
                 free(iface_data);
             }
@@ -499,10 +523,15 @@ static int32_t f_iface_close(t_x502_hnd hnd) {
         }
 
         if (tcp_data->ioctl_mutex != OSSPEC_INVALID_MUTEX) {
-            osspec_mutex_release(tcp_data->ioctl_mutex);
+            osspec_mutex_destroy(tcp_data->ioctl_mutex);
             tcp_data->ioctl_mutex = OSSPEC_INVALID_MUTEX;
         }
 
+        if (tcp_data->data_mutex != OSSPEC_INVALID_MUTEX) {
+            osspec_mutex_destroy(tcp_data->data_mutex);
+            tcp_data->data_mutex = OSSPEC_INVALID_MUTEX;
+        }
+
 
         free(hnd->iface_data);
         hnd->iface_data = NULL;
@@ -512,26 +541,58 @@ static int32_t f_iface_close(t_x502_hnd hnd) {
 
 static int32_t f_iface_stream_cfg(t_x502_hnd hnd, uint32_t ch, t_x502_stream_ch_params *params) {
     int32_t err = X502_ERR_OK;
-    t_tcp_iface_data *tcp_data = (t_tcp_iface_data*)hnd->iface_data;
-    if (tcp_data->data_sock==INVALID_SOCKET) {
-        err = f_iface_gen_ioctl(hnd, E502_CM4_CMD_DROP_DATA_CON, 0, NULL, 0, NULL, 0, NULL, 0);
-        if (err == X502_ERR_OK)
-            err = f_con_sock(&tcp_data->data_sock, tcp_data->ip_addr, tcp_data->data_port, tcp_data->open_tout);
-    }
 
-    if (err==X502_ERR_OK) {
-        unsigned buf_size = params->buf_size*4;
-        if (ch==X502_STREAM_CH_IN) {
-            tcp_data->recv_part_size = 0;
-            if (setsockopt(tcp_data->data_sock, SOL_SOCKET, SO_RCVBUF, (char*)&buf_size, sizeof(buf_size))==SOCKET_ERROR) {
-                err = X502_ERR_SOCKET_SET_BUF_SIZE;
+    t_tcp_iface_data *tcp_data = (t_tcp_iface_data*)hnd->iface_data;
+    err = osspec_mutex_lock(tcp_data->data_mutex, X502_MUTEX_TCP_DATA_LOCK_TOUT);
+    if (err == X502_ERR_OK) {
+        if (tcp_data->data_sock == INVALID_SOCKET) {
+            err = f_iface_gen_ioctl(hnd, E502_CM4_CMD_DROP_DATA_CON, 0, NULL, 0, NULL, 0, NULL, 0);
+            if (err == X502_ERR_OK) {
+                err = f_con_sock(&tcp_data->data_sock, tcp_data->ip_addr, tcp_data->data_port, tcp_data->open_tout);
             }
-        } else {
-            tcp_data->send_part_size = 0;
-            if (setsockopt(tcp_data->data_sock, SOL_SOCKET, SO_SNDBUF, (char*)&buf_size, sizeof(buf_size))==SOCKET_ERROR) {
-                err = X502_ERR_SOCKET_SET_BUF_SIZE;
+
+            if (err == X502_ERR_OK) {
+                int flag = 1;
+                if (setsockopt(tcp_data->data_sock, IPPROTO_TCP, TCP_NODELAY, (char*)&flag, sizeof(flag))==SOCKET_ERROR)
+                    err = X502_ERR_SOCKET_OPEN;
             }
         }
+
+        if (err == X502_ERR_OK) {
+            unsigned buf_size = params->buf_size*4;
+            if (ch==X502_STREAM_CH_IN) {
+                tcp_data->recv_part_size = 0;
+                if (setsockopt(tcp_data->data_sock, SOL_SOCKET, SO_RCVBUF, (char*)&buf_size, sizeof(buf_size))==SOCKET_ERROR) {
+                    err = X502_ERR_SOCKET_SET_BUF_SIZE;
+                }
+                else {
+                    dprintf("set SO_RCVBUF to %d\n", buf_size);
+                }
+#ifndef _WIN32
+                setsockopt(tcp_data->data_sock, SOL_SOCKET, SO_RCVBUFFORCE, (char*)&buf_size, sizeof(buf_size));
+#endif
+                socklen_t opt_len = sizeof(buf_size);
+                if (getsockopt(tcp_data->data_sock, SOL_SOCKET, SO_RCVBUF, (char*)&buf_size, &opt_len) != SOCKET_ERROR) {
+                    dprintf("get SO_RCVBUF = %d\n", buf_size);
+                }
+#ifdef _WIN32
+                else {
+                    dprintf("getsockopt error = %d\n", WSAGetLastError());
+                }
+#endif
+            } else {
+                tcp_data->send_part_size = 0;
+                if (setsockopt(tcp_data->data_sock, SOL_SOCKET, SO_SNDBUF, (char*)&buf_size, sizeof(buf_size))==SOCKET_ERROR) {
+                    err = X502_ERR_SOCKET_SET_BUF_SIZE;
+                }
+            }
+        }
+
+        if (err == X502_ERR_OK) {
+            tcp_data->data_chs_en |= (1UL << ch);
+        }
+
+        osspec_mutex_release(tcp_data->data_mutex);
     }
 
     return err;
@@ -556,20 +617,26 @@ static int32_t f_iface_stream_stop(t_x502_hnd hnd, uint32_t ch, uint32_t flags)
         err = hnd->iface_hnd->stream_running(hnd, ch, &running);
         if (!err && running) {
             err = f_iface_gen_ioctl(hnd, E502_CM4_CMD_STREAM_STOP, (ch << 16),
-                                      NULL, 0, NULL, 0, NULL, 0);       
+                                      NULL, 0, NULL, 0, NULL, 0);
         }
     }
     return err;
 }
 
 static int32_t f_iface_stream_free(t_x502_hnd hnd, uint32_t ch, uint32_t flags) {
-    int32_t err =  hnd->iface_hnd->stream_stop(hnd, ch, flags);
+    t_tcp_iface_data *tcp_data = (t_tcp_iface_data *)hnd->iface_data;
+    int32_t err = osspec_mutex_lock(tcp_data->data_mutex, X502_MUTEX_TCP_DATA_LOCK_TOUT);
     if (err == X502_ERR_OK) {
-        t_tcp_iface_data *tcp_data = (t_tcp_iface_data *)hnd->iface_data;
-        if (tcp_data->data_sock!=INVALID_SOCKET) {
-            closesocket(tcp_data->data_sock);
-            tcp_data->data_sock = INVALID_SOCKET;
+        err =  hnd->iface_hnd->stream_stop(hnd, ch, flags);
+        if (err == X502_ERR_OK) {
+            tcp_data->data_chs_en &= ~(1UL << ch);
+
+            if ((tcp_data->data_chs_en == 0) && (tcp_data->data_sock != INVALID_SOCKET)) {
+                closesocket(tcp_data->data_sock);
+                tcp_data->data_sock = INVALID_SOCKET;
+            }
         }
+        osspec_mutex_release(tcp_data->data_mutex);
     }
     return err;
 }
@@ -687,7 +754,7 @@ static int32_t f_iface_stream_get_rdy_cnt(t_x502_hnd hnd, uint32_t ch, uint32_t
 }
 
 
-int32_t e502_make_tcp_rec(t_x502_devrec *devrec, uint32_t flags, uint32_t tout) {
+int32_t e502_make_tcp_rec(t_x502_devrec *devrec, uint32_t flags, uint32_t tout, char const *devname) {
     int32_t err = (devrec == NULL) ? X502_ERR_INVALID_DEVICE_RECORD : X502_ERR_OK;
 
     X502_DevRecordInit(devrec);
@@ -699,7 +766,7 @@ int32_t e502_make_tcp_rec(t_x502_devrec *devrec, uint32_t flags, uint32_t tout)
         if ((devinfo_data==NULL) || (devinfo_ptr == NULL)) {
             err = X502_ERR_MEMORY_ALLOC;
         } else {
-            strcpy(devrec->devname, E502_DEVICE_NAME);
+            strcpy(devrec->devname, devname);
             devinfo_data->cmd_port = E502_TCP_DEFAULT_CMD_PORT;
             devinfo_data->data_port = E502_TCP_DEFAULT_DATA_PORT;
             devinfo_data->open_tout = tout;
@@ -721,9 +788,9 @@ int32_t e502_make_tcp_rec(t_x502_devrec *devrec, uint32_t flags, uint32_t tout)
     return err;
 }
 
-X502_EXPORT(int32_t) E502_MakeDevRecordByIpAddr(t_x502_devrec *devrec, uint32_t ip_addr,
-                                               uint32_t flags, uint32_t tout) {
-    int32_t err = e502_make_tcp_rec(devrec, flags, tout);
+X502_EXPORT(int32_t) E502_MakeDevRecordByIpAddr2(t_x502_devrec *devrec, uint32_t ip_addr,
+                                               uint32_t flags, uint32_t tout, char const *devname) {
+    int32_t err = e502_make_tcp_rec(devrec, flags, tout, devname);
     if (err == X502_ERR_OK) {
         t_tcp_devinfo_data *devinfo_data = (t_tcp_devinfo_data *)devrec->internal->iface_data;
         devinfo_data->ip_addr = ip_addr;
@@ -742,6 +809,11 @@ X502_EXPORT(int32_t) E502_MakeDevRecordByIpAddr(t_x502_devrec *devrec, uint32_t
     return err;
 }
 
+X502_EXPORT(int32_t) E502_MakeDevRecordByIpAddr(t_x502_devrec *devrec, uint32_t ip_addr,
+                                               uint32_t flags, uint32_t tout) {
+    return E502_MakeDevRecordByIpAddr2(devrec, ip_addr, flags, tout, E502_DEVICE_NAME);
+}
+
 X502_EXPORT(int32_t) E502_EthDevRecordSetCmdPort(t_x502_devrec *devrec, uint16_t cmd_port) {
     int32_t err = ((devrec == NULL) || (devrec->internal->iface != &f_tcp_iface)) ?
                 X502_ERR_INVALID_DEVICE_RECORD : X502_ERR_OK;
@@ -762,6 +834,19 @@ X502_EXPORT(int32_t) E502_EthDevRecordSetDataPort(t_x502_devrec *devrec, uint16_
     return err;
 }
 
+X502_EXPORT(int32_t) E16_OpenByIpAddr(t_x502_hnd hnd, uint32_t ip_addr, uint32_t flags, uint32_t tout) {
+    int32_t err = X502_CHECK_HND(hnd);
+    if (err == X502_ERR_OK) {
+        t_x502_devrec devinfo;
+        err = E502_MakeDevRecordByIpAddr2(&devinfo, ip_addr, flags, tout, E16_DEVICE_NAME);
+        if (err == X502_ERR_OK) {
+            err = X502_OpenByDevRecord(hnd, &devinfo);
+
+            X502_FreeDevRecordList(&devinfo, 1);
+        }
+    }
+    return err;
+}
 
 X502_EXPORT(int32_t) E502_OpenByIpAddr(t_x502_hnd hnd, uint32_t ip_addr, uint32_t flags, uint32_t tout) {
     int32_t err = X502_CHECK_HND(hnd);
@@ -790,3 +875,28 @@ X502_EXPORT(int32_t) E502_GetIpAddr(t_x502_hnd hnd, uint32_t *ip_addr) {
     }
     return err;
 }
+#else
+#include "e502api.h"
+X502_EXPORT(int32_t) E502_MakeDevRecordByIpAddr(t_x502_devrec *devrec, uint32_t ip_addr,
+                                               uint32_t flags, uint32_t tout) {
+    return  X502_ERR_NOT_IMPLEMENTED;
+}
+
+X502_EXPORT(int32_t) E502_EthDevRecordSetCmdPort(t_x502_devrec *devrec, uint16_t cmd_port) {
+    return  X502_ERR_NOT_IMPLEMENTED;
+}
+
+X502_EXPORT(int32_t) E502_EthDevRecordSetDataPort(t_x502_devrec *devrec, uint16_t data_port) {
+    return  X502_ERR_NOT_IMPLEMENTED;
+}
+
+
+X502_EXPORT(int32_t) E502_OpenByIpAddr(t_x502_hnd hnd, uint32_t ip_addr, uint32_t flags, uint32_t tout) {
+    return  X502_ERR_NOT_IMPLEMENTED;
+}
+
+
+X502_EXPORT(int32_t) E502_GetIpAddr(t_x502_hnd hnd, uint32_t *ip_addr) {
+    return  X502_ERR_NOT_IMPLEMENTED;
+}
+#endif
