@@ -14,7 +14,7 @@ typedef struct {
 #endif
 } t_tcp_devinfo_data;
 
-int32_t e502_make_tcp_rec(t_x502_devrec *devrec, uint32_t flags, uint32_t tout);
+int32_t e502_make_tcp_rec(t_x502_devrec *devrec, uint32_t flags, uint32_t tout, char const *devname);
 int32_t e502_svc_fill_devinfo(t_tcp_devinfo_data *data);
 
 #endif // E502API_TCP_PRIVATE_H
