@@ -1,3 +1,4 @@
+#ifdef ENABLE_USB
 #include "libusb-1.0/libusb.h"
 #include "e502api_private.h"
 #include "lboot_req.h"
@@ -12,6 +13,12 @@
 #define E502_USB_PID        0xE502
 #define E502_USB_REQ_TOUT   3000
 
+#define E440_USB_VID        0x0471
+#define E440_USB_PID        0x0440
+
+#define E16_USB_VID        0x2A52
+#define E16_USB_PID        0x0E16
+
 #define USB_CTL_REQ_MAX_SIZE 512
 
 
@@ -405,7 +412,7 @@ static OSSPEC_THREAD_FUNC_RET OSSPEC_THREAD_FUNC_CALL f_usb_rx_thread_func(void
             int wt_buf_rdy = 0;
 
 
-            osspec_mutex_lock(info->mutex, MUTEX_STREAM_LOCK_TOUT);           
+            osspec_mutex_lock(info->mutex, MUTEX_STREAM_LOCK_TOUT);
             if (info->rx.cpls[cpl_pos].state != RX_STATE_IDLE) {
                 wt_buf_rdy = 1;
                 osspec_event_clear(info->usb_wake_evt);
@@ -414,12 +421,17 @@ static OSSPEC_THREAD_FUNC_RET OSSPEC_THREAD_FUNC_CALL f_usb_rx_thread_func(void
 
 
             if (transf_completed[transf_check_pos] != TRANSF_CPL_IDLE) {
+                int lusberr;
                 /* иначе ждем событий от USB */
                 struct timeval tv;
                 tv.tv_sec =0;
                 tv.tv_usec = 200 * 1000;
-                if (libusb_handle_events_timeout_completed(NULL, &tv, (int*)&transf_completed[transf_check_pos]) != 0)
+                lusberr = libusb_handle_events_timeout_completed(NULL, &tv, (int*)&transf_completed[transf_check_pos]);
+
+                if ((lusberr != LIBUSB_SUCCESS) && (lusberr != LIBUSB_ERROR_INTERRUPTED)) {
                     err = X502_ERR_RECV;
+                }
+
             } else if (wt_buf_rdy) {
                 /* если буфер занят - ждем события от потока чтения */
                 osspec_event_wait(info->usb_wake_evt, OSSPEC_TIMEOUT_INFINITY);
@@ -539,7 +551,7 @@ static OSSPEC_THREAD_FUNC_RET OSSPEC_THREAD_FUNC_CALL f_usb_tx_thread_func(void
 
             transf_completed[transf_pos] = TRANSF_CPL_BUSY;
             if (libusb_submit_transfer(transfers[transf_pos])==0) {
-                osspec_mutex_lock(info->mutex, MUTEX_STREAM_LOCK_TOUT);                
+                osspec_mutex_lock(info->mutex, MUTEX_STREAM_LOCK_TOUT);
                 info->tx.busy_size+=snd_size;
                 osspec_mutex_release(info->mutex);
                 if (++transf_pos == USB_BULK_TX_MAX_TRANSF_CNT)
@@ -608,6 +620,23 @@ static OSSPEC_THREAD_FUNC_RET OSSPEC_THREAD_FUNC_CALL f_usb_tx_thread_func(void
 }
 
 
+static void f_stream_init(uint32_t ch, t_transf_info *info) {
+    info->stop_req = 0;
+    info->err = 0;
+    if (ch==X502_STREAM_CH_IN) {
+        unsigned i;
+        info->rx.cpl_get_pos = 0;
+        info->rx.buf_get_rdy = 0;
+
+
+        for (i=0; i < info->rx.cpl_cnt; i++) {
+            info->rx.cpls[i].state = RX_STATE_IDLE;
+        }
+    }  else {
+        info->tx.buf_pos_put = 0;
+        info->tx.buf_put_rdy = info->buf_size;
+    }
+}
 
 static int32_t f_iface_stream_cfg(t_x502_hnd hnd, uint32_t ch, t_x502_stream_ch_params *params) {
     t_usb_iface_data *usb_data = (t_usb_iface_data *)hnd->iface_data;
@@ -618,9 +647,6 @@ static int32_t f_iface_stream_cfg(t_x502_hnd hnd, uint32_t ch, t_x502_stream_ch_
     if (info->data == NULL) {
         info->buf_size = params->buf_size;
 
-        info->stop_req = 0;
-        info->err = 0;
-
 
         if (ch==X502_STREAM_CH_IN) {
             info->rx.transf_size = ((params->step+127)/128)*128;
@@ -634,20 +660,10 @@ static int32_t f_iface_stream_cfg(t_x502_hnd hnd, uint32_t ch, t_x502_stream_ch_
                 info->rx.cpl_cnt = USB_BULK_RX_MAX_TRANSF_CNT;
             info->buf_size = info->rx.cpl_cnt*info->rx.transf_size;
 
-            info->rx.cpl_get_pos = 0;
-            info->rx.buf_get_rdy = 0;
             info->rx.cpls = malloc(info->rx.cpl_cnt * sizeof(info->rx.cpls[0]));
             if (info->rx.cpls == NULL) {
                 err = X502_ERR_MEMORY_ALLOC;
-            } else {
-                unsigned i;
-                for (i=0; i < info->rx.cpl_cnt; i++) {
-                    info->rx.cpls[i].state = RX_STATE_IDLE;
-                }
             }
-        } else {
-            info->tx.buf_pos_put = 0;
-            info->tx.buf_put_rdy = info->buf_size;
         }
 
         if (err == X502_ERR_OK) {
@@ -657,15 +673,18 @@ static int32_t f_iface_stream_cfg(t_x502_hnd hnd, uint32_t ch, t_x502_stream_ch_
             }
         }
 
+        if (err == X502_ERR_OK) {
+            f_stream_init(ch, info);
+        }
 
-        if (!err) {
+        if (err == X502_ERR_OK) {
             uint16_t send_step = params->step > 0xFFFF ? 0xFFFF : params->step;
             err = f_ioreq(usb_data->devhnd, E502_CM4_CMD_STREAM_SET_STEP,
                           (ch<<16) | send_step, NULL, 0, NULL, 0, NULL, 0);
         }
 
 
-        if (err) {
+        if (err != X502_ERR_OK) {
             f_iface_stream_free(hnd, ch, 0);
         }
     }
@@ -681,10 +700,11 @@ static int32_t f_iface_stream_cfg(t_x502_hnd hnd, uint32_t ch, t_x502_stream_ch_
 
 static int32_t f_iface_stream_start(t_x502_hnd hnd, uint32_t ch, uint32_t flags) {
     t_usb_iface_data *usb_data = (t_usb_iface_data *)hnd->iface_data;
-    int err = 0;
+    int32_t err = X502_ERR_OK;
 
-    if (!err) {
+    if (err == X502_ERR_OK) {
         t_transf_info *info = &usb_data->streams[ch];
+        f_stream_init(ch, info);
         info->thread = osspec_thread_create(ch == X502_STREAM_CH_IN ?
                                                 f_usb_rx_thread_func : f_usb_tx_thread_func,
                                             usb_data, 0);
@@ -772,7 +792,7 @@ static int32_t f_iface_stream_read(t_x502_hnd hnd, uint32_t *buf, uint32_t size,
 
                 if (cur_state == RX_STATE_ERR) {
                     err = X502_ERR_RECV;
-                } else if (cur_state == RX_STATE_RDY){
+                } else if (cur_state == RX_STATE_RDY) {
                     osspec_mutex_lock(info->mutex, MUTEX_STREAM_LOCK_TOUT);
 
                     if (cpl->size) {
@@ -1014,9 +1034,8 @@ static int32_t f_fill_devlist(libusb_device_handle *hnd, t_x502_devrec *info) {
 
 
 
-
-X502_EXPORT(int32_t) E502_UsbGetDevRecordsList(t_x502_devrec *list, uint32_t size,
-                                             uint32_t flags, uint32_t *devcnt) {
+X502_EXPORT(int32_t) E502_UsbGetDevRecordsList2(t_x502_devrec *list, uint32_t size,
+                                             uint32_t flags, uint32_t *devcnt, uint16_t idVendor, uint16_t idProduct) {
     uint32_t curcnt=0;
     int32_t err = X502_ERR_OK;
     libusb_device **usb_devlist;
@@ -1040,8 +1059,8 @@ X502_EXPORT(int32_t) E502_UsbGetDevRecordsList(t_x502_devrec *list, uint32_t siz
 
         if (libusb_get_device_descriptor (usb_devlist[i], &devdescr) == 0) {
             //сравниваем VID, PID с индентификаторами загрузочного устройства
-            if ((devdescr.idVendor == E502_USB_VID)
-                    && (devdescr.idProduct == E502_USB_PID)) {
+            if ((devdescr.idVendor == idVendor)
+                    && (devdescr.idProduct == idProduct)) {
                 int32_t cur_err;
                 cur_err = libusb_open(usb_devlist[i], &usbhnd);
                 if (cur_err == X502_ERR_OK) {
@@ -1082,6 +1101,16 @@ X502_EXPORT(int32_t) E502_UsbGetDevRecordsList(t_x502_devrec *list, uint32_t siz
     return err != X502_ERR_OK ? err  : curcnt > (uint32_t)size ? (int32_t)size : (int32_t)curcnt ;
 }
 
+X502_EXPORT(int32_t) E440_UsbGetDevRecordsList(t_x502_devrec* list, uint32_t size,
+                                             uint32_t flags, uint32_t* devcnt) {
+    return E502_UsbGetDevRecordsList2(list, size, flags, devcnt, E440_USB_VID, E440_USB_PID);
+}
+
+X502_EXPORT(int32_t) E502_UsbGetDevRecordsList(t_x502_devrec *list, uint32_t size,
+                                             uint32_t flags, uint32_t *devcnt) {
+    return E502_UsbGetDevRecordsList2(list, size, flags, devcnt, E502_USB_VID, E502_USB_PID);
+}
+
 X502_EXPORT(int32_t) E502_OpenUsb(t_x502_hnd hnd, const char* serial) {
     return X502_Open(hnd, serial, E502_DEVICE_NAME, E502_UsbGetDevRecordsList);
 }
@@ -1092,6 +1121,32 @@ X502_EXPORT(int32_t) E502_UsbGetSerialList(char serials[][X502_SERIAL_SIZE], uin
 }
 
 
+X502_EXPORT(int32_t) E16_UsbGetDevRecordsList(t_x502_devrec *list, uint32_t size,
+                                             uint32_t flags, uint32_t *devcnt) {
+    return E502_UsbGetDevRecordsList2(list, size, flags, devcnt, E16_USB_VID, E16_USB_PID);
+}
+
+X502_EXPORT(int32_t) E16_OpenUsb(t_x502_hnd hnd, const char* serial) {
+    return X502_Open(hnd, serial, E16_DEVICE_NAME, E16_UsbGetDevRecordsList);
+}
+
+X502_EXPORT(int32_t) E16_UsbGetSerialList(char serials[][X502_SERIAL_SIZE], uint32_t size,
+                           uint32_t flags, uint32_t *devcnt) {
+   return X502_GetSerialList(serials, size, flags, devcnt, E16_DEVICE_NAME, E16_UsbGetDevRecordsList);
+}
+#else
+#include "e502api.h"
+X502_EXPORT(int32_t) E502_OpenUsb(t_x502_hnd hnd, const char* serial) {
+    return X502_ERR_NOT_IMPLEMENTED;
+}
+
+X502_EXPORT(int32_t) E502_UsbGetSerialList(char serials[][X502_SERIAL_SIZE], uint32_t size,
+                           uint32_t flags, uint32_t *devcnt) {
+   return X502_ERR_NOT_IMPLEMENTED;
+}
+#endif
+
+
 
 
 
