@@ -1,5 +1,5 @@
 /* Данный пример демонстрирует возможность установки циклического сигнала на 
-   вывод в модуль L502 (ввод не используется вообще).
+   вывод в модуль E16 / E502 / L502 (ввод не используется вообще).
 
    В примере определена таблица сигналов, которая задает несколько
    комбинаций сигналов для примера. Каждый сигнал задается количеством точек
@@ -42,6 +42,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
+#include "dev_funcs.h"
 
 #define TCP_CONNECTION_TOUT 5000
 #define OUT_SIGNAL_SIZE           2000
@@ -76,6 +77,11 @@ static double f_gen_sin(uint32_t cntr, uint32_t total_size, double amp) {
     return amp*sin(2*M_PI*cntr/total_size);
 }
 
+/* генерация тестового честчика */
+static double f_gen_cntr(uint32_t cntr, uint32_t total_size, double amp) {
+    return cntr + 0x123;
+}
+
 /* генерация синуса с периодом в два раза меньше периоду определяемому размером буфера */
 static double f_gen_sin2(uint32_t cntr, uint32_t total_size, double amp) {
     return amp*sin(2*2*M_PI*cntr/total_size);
@@ -93,6 +99,7 @@ static uint32_t f_gen_dout_cntr(uint32_t cntr, uint32_t total_size) {
 
 
 /* таблица, задающая сигналы для выдачи на вход */
+/* для модуля E16 частота выходного сигнала будет в 2 раза меньше, т.к. частота ЦАП 500КГц протип 1МГц у E502 */
 static t_sig_struct f_sig_tbl[] = {
     {100000, X502_DAC_RANGE, f_gen_saw, 0, NULL, NULL}, /* пила на одном канале с частотой 10 Гц */
     {2000, X502_DAC_RANGE, f_gen_sin, 0, NULL, NULL}, /* синусоидальный сигнал на одном канале с частотой 500 Гц */
@@ -202,153 +209,6 @@ static int32_t f_load_cycle_signal(t_x502_hnd hnd, int sig) {
     return err;
 }
 
-
-/* Функция находит все подключенные модули по интерфейсам PCI-Express и USB и
- * сохраняет записи о этих устройствах в выделенный массив.
- * Также создаются записи по переданным IP-адресам модулей и добавляются в конец
- * массива.
- * Указатель на выделенный массив, который должен быть потом очищен, сохраняется
- * в pdevrec_list, а количество действительных элементов (память которых должна
- * быть в дальнейшем освобождена с помощью X502_FreeDevRecordList()) возвращается
- * как результат функции */
-static uint32_t f_get_all_devrec(t_x502_devrec **pdevrec_list, uint32_t *ip_addr_list, unsigned ip_cnt) {
-    int32_t fnd_devcnt = 0;
-    uint32_t pci_devcnt = 0;
-    uint32_t usb_devcnt = 0;
-
-    t_x502_devrec *rec_list = NULL;
-
-    /* получаем количество подключенных устройств по интерфейсам PCI и USB */
-    L502_GetDevRecordsList(NULL, 0, 0, &pci_devcnt);
-    E502_UsbGetDevRecordsList(NULL, 0, 0, &usb_devcnt);
-
-    if ((pci_devcnt+usb_devcnt + ip_cnt) != 0) {
-        /* выделяем память для массива для сохранения найденного количества записей */
-        rec_list = malloc((pci_devcnt + usb_devcnt + ip_cnt) * sizeof(t_x502_devrec));
-
-        if (rec_list != NULL) {
-            unsigned i;
-            /* получаем записи о модулях L502, но не больше pci_devcnt */
-            if (pci_devcnt!=0) {
-                int32_t res = L502_GetDevRecordsList(&rec_list[fnd_devcnt], pci_devcnt, 0, NULL);
-                if (res >= 0) {
-                    fnd_devcnt += res;
-                }
-            }
-            /* добавляем записи о модулях E502, подключенных по USB, в конец массива */
-            if (usb_devcnt!=0) {
-                int32_t res = E502_UsbGetDevRecordsList(&rec_list[fnd_devcnt], usb_devcnt, 0, NULL);
-                if (res >= 0) {
-                    fnd_devcnt += res;
-                }
-            }
-
-            /* создаем записи для переданного массива ip-адресов */
-            for (i=0; i < ip_cnt; i++) {
-                if (E502_MakeDevRecordByIpAddr(&rec_list[fnd_devcnt], ip_addr_list[i],0, TCP_CONNECTION_TOUT) == X502_ERR_OK) {
-                    fnd_devcnt++;
-                }
-            }
-        }
-    }
-
-    if (fnd_devcnt != 0) {
-        /* если создана хотя бы одна запись, то сохраняем указатель на выделенный массив */
-        *pdevrec_list = rec_list;
-    } else {
-        *pdevrec_list = NULL;
-        free(rec_list);
-    }
-
-    return fnd_devcnt;
-}
-
-
-static t_x502_hnd f_dev_select_open(int argc, char** argv) {
-    t_x502_hnd hnd = NULL;
-    uint32_t fnd_devcnt,i, dev_ind;
-    t_x502_devrec *devrec_list = NULL;
-    uint32_t *ip_addr_list = NULL;
-    uint32_t ip_cnt = 0;
-
-    /* если есть аргументы командной строки, то предполагаем, что это могут быть
-       ip-адреса интересующих устройств. */
-    if (argc > 1) {
-        ip_addr_list = malloc((argc-1) * sizeof(ip_addr_list[0]));
-        if (ip_addr_list == NULL) {
-            fprintf(stderr, "Ошибка выделения памяти!\n");
-        } else {
-            for (i=1; (int)i < argc; i++) {
-                int a[4];
-                if (sscanf(argv[i], "%d.%d.%d.%d", &a[0], &a[1], &a[2], &a[3])==4) {
-                    ip_addr_list[ip_cnt++] = ((a[0] & 0xFF) << 24) |
-                                             ((a[1] & 0xFF) << 16) |
-                                             ((a[2] & 0xFF) <<  8) |
-                                             (a[3] & 0xFF);
-                }
-            }
-        }
-    }
-
-    /* получаем список модулей для выбора */
-    fnd_devcnt = f_get_all_devrec(&devrec_list, ip_addr_list, ip_cnt);
-
-    if (fnd_devcnt == 0) {
-        printf("Не найдено ни одного модуля\n");
-    } else {
-        /* выводим информацию по списку модулей */
-        printf("Доступны следующие модули:\n");
-        for (i=0; i < fnd_devcnt; i++) {
-            printf("Модуль № %d: %s, %-9s", i, devrec_list[i].devname,
-                   devrec_list[i].iface == X502_IFACE_PCI ? "PCI/PCIe" :
-                   devrec_list[i].iface == X502_IFACE_USB ? "USB" :
-                   devrec_list[i].iface == X502_IFACE_ETH ? "Ethernet" : "?");
-
-            /* при подключении по сети по IP-адресу серийный номер можно узнать
-               только после открытия соединения. При этом поле location
-               содержит строку с описанием адреса устройства */
-            if (devrec_list[i].iface != X502_IFACE_ETH) {
-                printf("Сер. номер: %s\n", devrec_list[i].serial);
-            } else {
-                printf("Адрес: %s\n", devrec_list[i].location);
-            }
-        }
-
-        /* выбираем нужный по введенному номеру модуля по порядку с клавиатуры */
-        printf("Введите номер модуля, с которым хотите работать (от 0 до %d)\n", fnd_devcnt-1);
-        fflush(stdout);
-        scanf("%d", &dev_ind);
-
-        if (dev_ind >= fnd_devcnt) {
-            printf("Неверно указан номер модуля...\n");
-        } else {
-            /* если ввели номер правильно - создаем описатель */
-            hnd = X502_Create();
-            if (hnd==NULL) {
-                fprintf(stderr, "Ошибка создания описателя модуля!");
-            } else {
-                /* устанавливаем связь с модулем по записи */
-                int32_t err = X502_OpenByDevRecord(hnd, &devrec_list[dev_ind]);
-                if (err != X502_ERR_OK) {
-                    fprintf(stderr, "Ошибка установления связи с модулем: %s!", X502_GetErrorString(err));
-                    X502_Free(hnd);
-                    hnd = NULL;
-                }
-            }
-        }
-
-        /* освобождение ресурсов действительных записей из списка */
-        X502_FreeDevRecordList(devrec_list, fnd_devcnt);
-        /* очистка памяти самого массива */
-        free(devrec_list);
-    }
-
-    /* освобождаем выделенный массив под IP-адреса (если был выделен) */
-    free(ip_addr_list);
-
-    return hnd;
-}
-
 int main(int argc, char **argv) {
     int32_t err = 0;
     uint32_t ver;
@@ -358,59 +218,27 @@ int main(int argc, char **argv) {
     /* устанавливаем локаль, чтобы можно было выводить по-русски в CP1251 без перевода в OEM */
     setlocale(LC_CTYPE, "");
 #endif
-    /* получаем версию библиотеки */
-    ver = X502_GetLibraryVersion();
-    printf("Верисия библиотеки: %d.%d.%d\n", (ver >> 24)&0xFF, (ver>>16)&0xFF, (ver>>8)&0xFF);
 
     /********** Получение списка устройств и выбор, с каким будем работать ******************/
-    hnd = f_dev_select_open(argc, argv);
+    hnd = select_dev_from_list(argc, argv, 0);
 
     /********************************** Работа с модулем **************************/
     /* если успешно выбрали модуль и установили с ним связь - продолжаем работу */
     if (hnd != NULL) {
-        /* получаем информацию */
-        t_x502_info info;
-        err = X502_GetDevInfo(hnd, &info);
+        /* Разрешаем все каналы на вывод (в этом примере мы не будем
+            использовать асинхронный вывод), а какие каналы реально будут
+            использованы определяем когда подаем данные в L502_PrepareData */
+        err = X502_StreamsEnable(hnd, X502_STREAM_ALL_OUT);
         if (err != X502_ERR_OK) {
-            fprintf(stderr, "Ошибка получения серийного информации о модуле: %s!", X502_GetErrorString(err));
-        } else {
-            /* выводим полученную информацию */
-            printf("Установлена связь со следующим модулем:\n");
-            printf(" Серийный номер          : %s\n", info.serial);
-            printf(" Наличие ЦАП             : %s\n", info.devflags & X502_DEVFLAGS_DAC_PRESENT ? "Да" : "Нет");
-            printf(" Наличие BlackFin        : %s\n", info.devflags & X502_DEVFLAGS_BF_PRESENT ? "Да" : "Нет");
-            printf(" Наличие гальваноразвязки: %s\n", info.devflags & X502_DEVFLAGS_GAL_PRESENT ? "Да" : "Нет");
-            printf(" Индустриальное исп.     : %s\n", info.devflags & X502_DEVFLAGS_INDUSTRIAL ? "Да" : "Нет");
-            printf(" Наличие интерф. PCI/PCIe: %s\n", info.devflags & X502_DEVFLAGS_IFACE_SUPPORT_PCI ? "Да" : "Нет");
-            printf(" Наличие интерф. USB     : %s\n", info.devflags & X502_DEVFLAGS_IFACE_SUPPORT_USB ? "Да" : "Нет");
-            printf(" Наличие интерф. Ethernet: %s\n", info.devflags & X502_DEVFLAGS_IFACE_SUPPORT_ETH ? "Да" : "Нет");
-            printf(" Версия ПЛИС             : %d.%d\n", (info.fpga_ver >> 8) & 0xFF, info.fpga_ver & 0xFF);
-            printf(" Версия PLDA             : %d\n", info.plda_ver);
-            if (info.mcu_firmware_ver != 0) {
-                printf(" Версия прошивки ARM     : %d.%d.%d.%d\n",
-                       (info.mcu_firmware_ver >> 24) & 0xFF,
-                       (info.mcu_firmware_ver >> 16) & 0xFF,
-                       (info.mcu_firmware_ver >>  8) & 0xFF,
-                       info.mcu_firmware_ver & 0xFF);
-            }
-        }
-
-        if (err == X502_ERR_OK) {
-            /* Разрешаем все каналы на вывод (в этом примере мы не будем
-               использовать асинхронный вывод), а какие каналы реально будут
-               использованы определяем когда подаем данные в L502_PrepareData */
-            err = X502_StreamsEnable(hnd, X502_STREAM_ALL_OUT);
-            if (err != X502_ERR_OK) {
-                fprintf(stderr, "Ошибка разрешения потоков (%d): %s!", err,
-                        X502_GetErrorString(err));
-            }
+            fprintf(stderr, "Ошибка разрешения потоков (%d): %s!", err,
+                    X502_GetErrorString(err));
         }
 
         if (err == X502_ERR_OK) {
             /* если хотим, то можем загрузить синхнал до разрешения
              * синхронного ввода. Тогда мы могли бы его запустить, наприемер,
                всесте с АЦП по L502_StreamsStart() */
-            err = f_load_cycle_signal(hnd, 0);
+            err = f_load_cycle_signal(hnd, 1);
 
             /* разрешаем синхронный ввод. В принципе можно его разрешить только
              * после загрузки первого синала в нашем примере. Но данный пример
