@@ -1,7 +1,9 @@
 #ifndef CRC_H_
 #define CRC_H_
 
-
+#ifdef __cplusplus
+extern "C" {
+#endif
 /*===============================================================================================================*/
 extern unsigned short eval_crc16     (unsigned short crc, const unsigned char *msg, unsigned msg_len);
 extern unsigned short eval_crc16tbl  (unsigned short crc, const unsigned char *msg, unsigned msg_len);
@@ -12,4 +14,8 @@ extern unsigned char  eval_revcrc8   (unsigned char crc, const unsigned char *ms
 #endif
 /*===============================================================================================================*/
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
