@@ -3,26 +3,29 @@
   Модуль представляет собой набор функций для реализации таймера на основе
   системных часов процессора (контроллера) или ОС.
 
+  Работа таймера основана на системном счетчике, который должен равномерно
+  увеличиваться с течением времени.
   Реализация функций для инициализации системного счетчика и получения его значения
   реализуются для каждой платформы своим портом. Также порт определяет количество
   тиков системного счетчика за секунду с помощью константы #LCLOCK_TICKS_PER_SECOND
-  и тип @c t_lclock_ticks для представления текущего значения системного счетчика.
+  и тип #t_lclock_ticks для представления текущего значения системного счетчика.
   Эти функции могут использоваться как напрямую, так и через общее API для работы
   с таймером из ltimer.h, который может использоваться для отсчета интервалов
-  времени, например для таймаутов.
-  Все функции принимают значения в отсчетах системного клока, поэтому необходимо
-  использовать константу #LCLOCK_TICKS_PER_SECOND для времени в количество отсчетов.
-  Также можно использовать макросы LTIMER_MS_TO_CLOCK_TICKS() и LTIMER_CLOCK_TICKS_TO_MS()
-  для перевода интервалов в миллисекундах в количество тиков и наоборот.
+  времени.
 
-  Для работы с модулем в проекте нужно добавить в проект в стандартные пути 
-  для включения заголовочных файлов пути к файлу ltimer.h и к порту директории с портом
+  Все функции принимают значения в отсчетах системного счетчика, поэтому необходимо
+  использовать константу #LCLOCK_TICKS_PER_SECOND для перевода времени в количество отсчетов.
+  Также можно использовать макросы LTIMER_MS_TO_CLOCK_TICKS() и LTIMER_CLOCK_TICKS_TO_MS()
+  для перевода интервалов в миллисекундах в количество отсчетов и наоборот.
+
+  Для работы с модулем в проекте нужно добавить в стандартные пути
+  для включения заголовочных файлов пути к файлу ltimer.h и к директории с портом
   для используемой платформы, а также путь к файлу lcspec.h, в котором
   описаны специфические для компилятора макросы (в данном случае используется
-  LINLINE для определения inline-функций). Также необходимо в исходники 
+  #LINLINE для определения inline-функций). Также необходимо в исходники
   добавить файл lclock.c из нужного порта.
 
-  При работе с системным клоком контроллера перед вызовом функций данного модуля
+  При работе с системным счетчиком контроллера перед вызовом функций данного модуля
   необходимо вызвать функцию lclock_init() (или lclock_init_val(), если нужно задать
   начальное значение системного счетчика). При работе с системными часами ОС
   обычно эта функция не требуется, так как таймер ОС уже инициализирован
@@ -39,16 +42,17 @@
 
   При стандартной работе, сперва таймер устанавливается для отсчета заданного
   интервала с помощью ltimer_set(). Проверить, истек ли заданный интервал,
-  можно с помощью ltimer_expired(). Количество отсчетов системного таймера
+  можно с помощью ltimer_expired(). Количество отсчетов системного счетчика
   до истечения интервала можно получить с помощью ltimer_expiration().
   Перезапустить таймер на установленный в предыдущем ltimer_set() интервал можно
   с помощью ltimer_reset() или ltimer_restart() (относительно времени последнего
   истечения таймера или текущего времени соответственно).
 
   Например, если какие-то действия нужно выполнять в течении заданного интервала,
-  это можно сделать следующим образом:
+  то это можно сделать следующим образом:
 
   @code
+  //время интервала в мс
   #define TIMEOUT_MS   200
 
   void main(void) {
@@ -92,6 +96,38 @@ typedef struct ltimer {
 } t_ltimer;
 
 
+/***************************************************************************//**
+  @brief Получить время запуска таймера
+
+  @param[in] t          Указатель на состояние таймера
+  @return               Значение времени запуска таймера в системных клоках
+ ******************************************************************************/
+static LINLINE t_lclock_ticks ltimer_start_time(t_ltimer *t) {
+    return t->start;
+}
+
+/***************************************************************************//**
+  @brief Получить время срабатывания таймера
+
+  @param[in] t          Указатель на состояние таймера
+  @return               Значение времени срабатывания таймера в системных клоках
+ ******************************************************************************/
+static LINLINE t_lclock_ticks ltimer_expiration_time(t_ltimer *t) {
+    return t->start + t->interval;
+}
+
+/***************************************************************************//**
+  @brief Получить установленный интервал для таймера
+
+  @param[in] t          Указатель на состояние таймера
+  @return               Значение установленного интервала в системных клоках
+ ******************************************************************************/
+static LINLINE t_lclock_ticks ltimer_interval(t_ltimer *t) {
+    return t->interval;
+}
+
+
+
 /***************************************************************************//**
   @brief Установить временной интервал относительно заданного времени
 
@@ -123,6 +159,21 @@ static LINLINE void ltimer_set(t_ltimer *t, t_lclock_ticks interval) {
     ltimer_set_at(t, interval, lclock_get_ticks());
 }
 
+/***************************************************************************//**
+  @brief Установить временной интервал в мс
+
+  Функция устанавливает таймер на заданный интервал в милисекундах относительно
+  текущего времени.
+  С помощью функции ltimer_expired() можно в дальнейшем проверить, истек ли
+  заданный интервал.
+  @param[in] t        Указатель на состояние таймера
+  @param[in] ms       Интервал, через который таймер истечет в милисекундах.
+ ******************************************************************************/
+static LINLINE void ltimer_set_ms(t_ltimer *t, unsigned ms) {
+    ltimer_set(t, LTIMER_MS_TO_CLOCK_TICKS(ms));
+}
+
+
 
 /***************************************************************************//**
   @brief Сброс таймера.
@@ -138,6 +189,22 @@ static LINLINE void ltimer_reset(t_ltimer *t) {
     t->start += t->interval;
 }
 
+
+
+/***************************************************************************//**
+  @brief Перезапуск таймера с заданного времени.
+
+  Данная функции перезапускает таймер на интервал, заданный в последнем ltimer_set(),
+  относительно текущего времени, начиная с указанного времени.
+  @param[in] t          Указатель на состояние таймера
+  @param[in] start_time Значение системного счетчика, соответствующего времени,
+                         с которого перезапускается таймер
+  *****************************************************************************/
+static LINLINE void ltimer_restart_at(t_ltimer *t, t_lclock_ticks time) {
+    t->start = time;
+}
+
+
 /***************************************************************************//**
   @brief Перезапуск таймера.
 
@@ -150,7 +217,7 @@ static LINLINE void ltimer_reset(t_ltimer *t) {
   @param[in] t        Указатель на состояние таймера  
   *****************************************************************************/
 static LINLINE void ltimer_restart(t_ltimer *t) {
-    t->start = lclock_get_ticks();
+    ltimer_restart_at(t, lclock_get_ticks());
 }
 
 /***************************************************************************//**
@@ -164,7 +231,7 @@ static LINLINE void ltimer_restart(t_ltimer *t) {
     @return             Не ноль, если интервал истечет, иначе --- ноль
  ******************************************************************************/
 static LINLINE int ltimer_expired_at(const t_ltimer *t, t_lclock_ticks exp_time) {
-    return (t_lclock_ticks)(exp_time - t->start) >= (t_lclock_ticks)t->interval;
+    return (t_lclock_ticks)(exp_time - t->start) > (t_lclock_ticks)t->interval;
 }
 
 /***************************************************************************//**
