@@ -4,9 +4,9 @@
 #include "x502api.h"
 
 
-#define OSSPEC_USE_MUTEX
-#define OSSPEC_USE_EVENTS
-#define OSSPEC_USE_THREADS
+//#define OSSPEC_USE_MUTEX
+//#define OSSPEC_USE_EVENTS
+//#define OSSPEC_USE_THREADS
 
 
 #define OSSPEC_ERR_MUTEX_INVALID_HANDLE     X502_ERR_MUTEX_INVALID_HANDLE
