@@ -6,9 +6,36 @@
 #include <stdlib.h>
 
 
-static const double f_scales[] = {10., 5., 2., 1., 0.5, 0.2};
+static const double f_x502_scales[] = {10., 5.,         2.,           1., 0.5, 0.2};
+// В E16 только 4 диапазона!
+static const double f_e16_scales[] = {10., 2.5, 0.625, 0.15625, 0., 0.};
 
 
+X502_EXPORT(double const*) E16_GetAdcRanges(uint32_t *ranges_len) {
+    if (ranges_len) {
+        *ranges_len = E16_ADC_RANGE_CNT;
+    }
+    return f_e16_scales;
+}
+
+X502_EXPORT(double const*) E502_GetAdcRanges(uint32_t *ranges_len) {
+    if (ranges_len) {
+        *ranges_len = X502_ADC_RANGE_CNT;
+    }
+    return f_x502_scales;
+}
+
+X502_EXPORT(double const*) X502_GetAdcRanges(t_x502_hnd hnd, uint32_t *ranges_len) {
+    int32_t err  = X502_CHECK_HND(hnd);
+    if (err != X502_ERR_OK) {
+        if (x502_is_E16(hnd)) {
+            return E16_GetAdcRanges(ranges_len);
+        } else {
+            return E502_GetAdcRanges(ranges_len);
+        }
+    }
+    return NULL;
+}
 
 
 X502_EXPORT(t_x502_hnd) X502_Create(void) {
@@ -67,14 +94,23 @@ X502_EXPORT(int32_t) X502_Close(t_x502_hnd hnd) {
     return err;
 }
 
+X502_EXPORT(int32_t) X502_IsOpened(t_x502_hnd hnd) {
+    int32_t err  = X502_CHECK_HND(hnd);
+    if ((err == X502_ERR_OK) && !(hnd->flags & PRIV_FLAGS_OPENED)) {
+        err = X502_ERR_DEVICE_NOT_OPENED;
+    }
+    return err;
+}
+
 
 
 X502_EXPORT(int32_t) X502_OpenByDevRecord(t_x502* hnd, const t_x502_devrec *devrec) {
     int32_t err  = X502_CHECK_HND(hnd);
     if ((err == X502_ERR_OK) && ((devrec==NULL) || (devrec->sign!=X502_DEVREC_SIGN)))
         err = X502_ERR_INVALID_DEVICE_RECORD;
-    if ((err == X502_ERR_OK) && (hnd->flags & PRIV_FLAGS_OPENED))
-        err = X502_ERR_ALREADY_OPENED;
+    if (X502_IsOpened(hnd) == X502_ERR_OK) {
+        X502_Close(hnd);
+    }
     if (err == X502_ERR_OK) {
         hnd->iface_hnd = (const t_x502_dev_iface *)(devrec->internal->iface);
         memcpy(hnd->info.serial, devrec->serial, X502_SERIAL_SIZE);
@@ -85,18 +121,36 @@ X502_EXPORT(int32_t) X502_OpenByDevRecord(t_x502* hnd, const t_x502_devrec *devr
         if (err == X502_ERR_OK) {
             X502_SetLChannel(hnd, 0, 0, X502_LCH_MODE_COMM, X502_ADC_RANGE_10, 0);
             hnd->set.lch_cnt = 1;
-            hnd->set.adc_freq_div = 1;
             hnd->set.adc_frame_delay = 0;
             hnd->set.sync_mode = X502_SYNC_INTERNAL;
             hnd->set.sync_start_mode = X502_SYNC_INTERNAL;
 
-            hnd->set.din_freq_div = 1;
-            hnd->set.out_freq_div = 2;
 
-            hnd->set.ref_freq = X502_REF_FREQ_2000KHZ;
             hnd->set.ext_ref_freq = 0;
             hnd->streams = 0;
 
+            if (x502_is_E16(hnd)) {
+                hnd->set.ref_freq = E16_REF_FREQ_48000KHZ;
+                hnd->set.dac_range = E16_DAC_RANGE;
+                hnd->set.dac_code_max = E16_DAC_SCALE_CODE_MAX;
+                hnd->set.adc_code_max = E16_ADC_SCALE_CODE_MAX;
+                hnd->set.f_scales = f_e16_scales;
+
+                hnd->set.adc_freq_div = E16_REF_FREQ_48000KHZ / E16_ADC_FREQ_DEFAULT;
+                hnd->set.din_freq_div = E16_REF_FREQ_48000KHZ / E16_DIN_FREQ_DEFAULT;
+                hnd->set.out_freq_div = E16_REF_FREQ_48000KHZ / E16_OUT_FREQ_DEFAULT;
+            } else {
+                hnd->set.ref_freq = X502_REF_FREQ_2000KHZ;
+                hnd->set.dac_range = X502_DAC_RANGE;
+                hnd->set.dac_code_max = X502_DAC_SCALE_CODE_MAX;
+                hnd->set.adc_code_max = X502_ADC_SCALE_CODE_MAX;
+                hnd->set.f_scales = f_x502_scales;
+
+                hnd->set.adc_freq_div = 1;
+                hnd->set.din_freq_div = 1;
+                hnd->set.out_freq_div = 2;
+            }
+
             hnd->flags = PRIV_FLAGS_OPENED;
 
 
@@ -133,10 +187,16 @@ X502_EXPORT(int32_t) X502_OpenByDevRecord(t_x502* hnd, const t_x502_devrec *devr
                             hnd->mode = X502_MODE_FPGA;
                         }
 
-                        /** @todo Для BlackFin проверить наличие прошивки */
 
                         if (hnd->mode==X502_MODE_DSP) {
-                            err = hnd->iface_hnd->fpga_reg_write(hnd, X502_REGS_BF_CMD, X502_BF_CMD_HDMA_RST);
+                            /* если blackfin находится в состоянии сброса, то возвращаемся в режим
+                               FPGA, т.к. все команды к HostDMA все равно не выполнятся */
+                            if (!(bf_ctl & X502_REGBIT_BF_CTL_BF_RESET_Msk)) {
+                                err = X502_SetMode(hnd, X502_MODE_FPGA);
+                            } else {
+                                err = hnd->iface_hnd->fpga_reg_write(hnd, X502_REGS_BF_CMD, X502_BF_CMD_HDMA_RST);
+                                /** @todo Для BlackFin проверить наличие прошивки */
+                            }
                         }
                     }
 
@@ -192,7 +252,7 @@ X502_EXPORT(int32_t) X502_OpenByDevRecord(t_x502* hnd, const t_x502_devrec *devr
         }
         /* читаем информацию из EEPROM (в первую очередь -
            калибровочные коэффициенты) */
-        if (err == X502_ERR_OK) {
+        if (err == X502_ERR_OK || err == X502_ERR_FPGA_NOT_LOADED) {
             int i;
 
             for (i=0; i < X502_ADC_RANGE_CNT; i++) {
@@ -402,7 +462,6 @@ X502_EXPORT(int32_t) X502_ProcessData(t_x502_hnd hnd, const uint32_t *src, uint3
                                       din_data, din_data_size, NULL, NULL);
 }
 
-
 X502_EXPORT(int32_t) X502_ProcessDataWithUserExt(t_x502_hnd hnd, const uint32_t* src, uint32_t size,
                                    uint32_t flags, double *adc_data,
                                    uint32_t *adc_data_size, uint32_t *din_data, uint32_t *din_data_size,
@@ -475,7 +534,7 @@ X502_EXPORT(int32_t) X502_ProcessDataWithUserExt(t_x502_hnd hnd, const uint32_t*
                     if (err == X502_ERR_OK) {
                         double res_val = val;
                         if (flags & X502_PROC_FLAGS_VOLT) {
-                            res_val = f_scales[range]*res_val/X502_ADC_SCALE_CODE_MAX;
+                            res_val = hnd->set.f_scales[range]*res_val/hnd->set.adc_code_max;
                         }
 
 
@@ -511,6 +570,8 @@ X502_EXPORT(int32_t) X502_ProcessDataWithUserExt(t_x502_hnd hnd, const uint32_t*
                     usr_cnt++;
                 }
                 break;
+            default:
+                break;
             }
         }
 
@@ -527,15 +588,21 @@ X502_EXPORT(int32_t) X502_ProcessDataWithUserExt(t_x502_hnd hnd, const uint32_t*
 }
 
 uint32_t prepare_dac_wrd(t_x502_hnd hnd, double val, uint32_t flags, const t_x502_cbr_coef* coef) {
-    int32_t wrd = 0;
+    uint16_t wrd = 0;
     if (flags & X502_DAC_FLAGS_VOLT) {
-        val = (val/X502_DAC_RANGE)*X502_DAC_SCALE_CODE_MAX;
+        val = (val / hnd->set.dac_range) * hnd->set.dac_code_max;
     }
     if (flags & X502_DAC_FLAGS_CALIBR) {
-        val = (val+coef->offs)*coef->k;
+        val = (val + coef->offs) * coef->k;
+    }
+    if (val > SHRT_MAX) {
+        wrd = SHRT_MAX;
+    } else
+    if (val < SHRT_MIN) {
+        wrd = SHRT_MIN;
+    } else {
+        wrd = val;
     }
-    wrd = (int32_t)val;
-    wrd &= 0xFFFF;
     return wrd;
 }
 
@@ -581,7 +648,7 @@ X502_EXPORT(int32_t) X502_FpgaRegWrite(t_x502_hnd hnd, uint32_t reg, uint32_t va
 X502_EXPORT(int32_t) X502_FpgaRegRead(t_x502_hnd hnd, uint32_t reg, uint32_t *val) {
     int32_t err = X502_CHECK_HND_OPENED(hnd);
     if (err == X502_ERR_OK) {
-        err = bf_fpga_reg_rd(hnd, reg, val);
+        err = hnd->iface_hnd->fpga_reg_read(hnd, reg, val);
     }
     return err;
 }
@@ -603,7 +670,7 @@ X502_EXPORT(int32_t) X502_DevRecordInit(t_x502_devrec *rec) {
 
 X502_EXPORT(int32_t) X502_LedBlink(t_x502_hnd hnd) {
     int32_t err = X502_CHECK_HND_OPENED(hnd);
-    if (err == X502_ERR_OK) {        
+    if (err == X502_ERR_OK) {
         int32_t err2;
         err = hnd->mode == X502_MODE_DSP ? bf_fpga_reg_wr(hnd, X502_REGS_IOHARD_LED, 0) :
                                            X502_FpgaRegWrite(hnd, X502_REGS_IOHARD_LED, 0);
@@ -632,6 +699,8 @@ X502_EXPORT(int32_t) X502_SetDigInPullup(t_x502_hnd hnd, uint32_t pullups) {
             val |= 0x10;
         if (pullups & X502_PULLDOWN_START_IN)
             val |= 0x20;
+        if (pullups & E16_MODE_RELAY_ON)
+            val |= 0x40;
 
         err = hnd->mode == X502_MODE_DSP ? bf_fpga_reg_wr(hnd, X502_REGS_IOHARD_DIGIN_PULLUP, val) :
                                            X502_FpgaRegWrite(hnd, X502_REGS_IOHARD_DIGIN_PULLUP, val);
