@@ -12,6 +12,13 @@
     && ((mode)!=X502_SYNC_DI_SYN2_FALL) \
     ? X502_ERR_INVALID_SYNC_MODE : X502_ERR_OK
 
+/** Максимальное значения частоты АЦП для варианта E16*/
+#define E16_ADC_FREQ_MAX        1000000
+/** Максимальное значения частоты DIN для варианта E16*/
+#define E16_DIN_FREQ_MAX        1000000
+/** Максимальное значения частоты OUT для варианта E16*/
+#define E16_OUT_FREQ_MAX        1000000
+
 static const uint16_t f_regadd_k[X502_ADC_RANGE_CNT] = {X502_REGS_IOARITH_K10,
                                                         X502_REGS_IOARITH_K5,
                                                         X502_REGS_IOARITH_K2,
@@ -27,6 +34,9 @@ static const uint16_t f_regadd_offs[X502_ADC_RANGE_CNT] = {X502_REGS_IOARITH_B10
                                                         X502_REGS_IOARITH_B02};
 
 
+static uint32_t f_get_out_freq_div_min(double ref_freq) {
+    return ref_freq > X502_OUT_FREQ_REF_LF_VAL ? X502_OUT_FREQ_DIV_MIN : X502_OUT_FREQ_DIV_MIN_REF_LF;
+}
 
 
 X502_EXPORT(int32_t) X502_SetLChannel(t_x502_hnd hnd, uint32_t lch, uint32_t phy_ch,
@@ -36,11 +46,15 @@ X502_EXPORT(int32_t) X502_SetLChannel(t_x502_hnd hnd, uint32_t lch, uint32_t phy
         err = X502_ERR_STREAM_IS_RUNNING;
 
     if (!err) {
+        /* Для измерения собственного нуля номер канала не имеет значения */
+        if (mode == X502_LCH_MODE_ZERO)
+            phy_ch = 0;
+
         if ((mode != X502_LCH_MODE_COMM) && (mode != X502_LCH_MODE_DIFF) &&
                 (mode != X502_LCH_MODE_ZERO)) {
             err = X502_ERR_INVALID_LCH_MODE;
         } else if ((phy_ch >= X502_ADC_COMM_CH_CNT) ||
-                   ((mode != X502_LCH_MODE_COMM) && (phy_ch >= X502_ADC_DIFF_CH_CNT))) {
+                   ((mode == X502_LCH_MODE_DIFF) && (phy_ch >= X502_ADC_DIFF_CH_CNT))) {
             err = X502_ERR_INVALID_LCH_PHY_NUMBER;
         } else if (range >= X502_ADC_RANGE_CNT) {
             err = X502_ERR_INVALID_LCH_RANGE;
@@ -66,6 +80,8 @@ X502_EXPORT(int32_t) X502_SetLChannelCount(t_x502_hnd hnd, uint32_t lch_cnt) {
     if (!err) {
         if (lch_cnt > X502_LTABLE_MAX_CH_CNT) {
             err = X502_ERR_INVALID_LTABLE_SIZE;
+        } else if (x502_is_E16(hnd) && lch_cnt > E16_LTABLE_MAX_CH_CNT) {
+            err = X502_ERR_INVALID_LTABLE_SIZE;
         } else {
             hnd->set.lch_cnt = lch_cnt;
         }
@@ -90,7 +106,9 @@ X502_EXPORT(int32_t) X502_SetAdcFreqDivider(t_x502_hnd hnd, uint32_t adc_freq_di
         err = X502_ERR_STREAM_IS_RUNNING;
 
     if (!err)  {
-        if ((adc_freq_div==0) || (adc_freq_div > X502_ADC_FREQ_DIV_MAX)) {
+        if ((adc_freq_div==0) ||
+            (x502_is_E16(hnd) && adc_freq_div > E16_ADC_FREQ_DIV_MAX) ||
+            (!x502_is_E16(hnd) && adc_freq_div > X502_ADC_FREQ_DIV_MAX)) {
             err = X502_ERR_INVALID_ADC_FREQ_DIV;
         } else {
             hnd->set.adc_freq_div = adc_freq_div;
@@ -119,7 +137,9 @@ X502_EXPORT(int32_t) X502_SetDinFreqDivider(t_x502_hnd hnd, uint32_t din_freq_di
         err = X502_ERR_STREAM_IS_RUNNING;
 
     if (!err) {
-        if ((din_freq_div==0) || (din_freq_div > X502_DIN_FREQ_DIV_MAX)) {
+        if ((din_freq_div==0) ||
+            (x502_is_E16(hnd) && din_freq_div > E16_DIN_FREQ_DIV_MAX) ||
+            (!x502_is_E16(hnd) && din_freq_div > X502_DIN_FREQ_DIV_MAX)) {
             err = X502_ERR_INVALID_ADC_FREQ_DIV;
         } else {
             hnd->set.din_freq_div = din_freq_div;
@@ -141,7 +161,11 @@ X502_EXPORT(int32_t) X502_SetOutFreqDivider(t_x502_hnd hnd, uint32_t out_freq_di
     }
 
     if (!err) {
-        if ((out_freq_div<X502_OUT_FREQ_DIV_MIN) || (out_freq_div > X502_OUT_FREQ_DIV_MAX)) {
+        double ref_freq;
+        X502_GetRefFreqValue(hnd, &ref_freq);
+        if ((out_freq_div < f_get_out_freq_div_min(ref_freq)) ||
+            (x502_is_E16(hnd) && out_freq_div > E16_OUT_FREQ_DIV_MAX) ||
+            (!x502_is_E16(hnd) && out_freq_div > X502_OUT_FREQ_DIV_MAX)) {
             err = X502_ERR_INVALID_OUT_FREQ_DIV;
         } else {
             hnd->set.out_freq_div = out_freq_div;
@@ -171,7 +195,9 @@ X502_EXPORT(int32_t) X502_SetMode(t_x502_hnd hnd, uint32_t mode) {
         if (!err) {
             val &= ~(X502_REGBIT_BF_CTL_CLK_DIV_Msk | X502_REGBIT_BF_CTL_DBG_MODE_Msk | X502_REGBIT_BF_CTL_DSP_MODE_Msk);
             if (mode==X502_MODE_DSP) {
-                val |= X502_REGBIT_BF_CTL_DSP_MODE_Msk;
+                /* в DSP режиме сброс должен быть снят с blackfin, иначе не будут работать
+                   команды по HostDMA */
+                val |= X502_REGBIT_BF_CTL_DSP_MODE_Msk | X502_REGBIT_BF_CTL_BF_RESET_Msk;
             } else if (mode==X502_MODE_DEBUG) {
                 val |= X502_REGBIT_BF_CTL_DBG_MODE_Msk;
             } else if (mode!=X502_MODE_FPGA) {
@@ -203,20 +229,35 @@ X502_EXPORT(int32_t) X502_GetMode(t_x502_hnd hnd, uint32_t* mode) {
     return err;
 }
 
-X502_EXPORT(int32_t) X502_CalcAdcFreq(double ref_freq, uint32_t lch_cnt, double *f_acq,
+#define IS_E16_FREQ(hnd, ref_freq) ((hnd && x502_is_E16(hnd)) || (uint32_t)ref_freq == E16_REF_FREQ_48000KHZ)
+
+X502_EXPORT(int32_t) X502_CalcAdcFreq2(t_x502_hnd hnd, double ref_freq, uint32_t lch_cnt, double *f_acq,
                                       double *f_frame, uint32_t *adc_freq_div, uint32_t *adc_frame_delay) {
     int32_t err = (f_acq==NULL) ? X502_ERR_INVALID_POINTER : X502_ERR_OK;
     if (err == X502_ERR_OK) {
         uint32_t cur_adc_freq_div, cur_frame_delay = 0;
+        uint32_t div_max = X502_ADC_FREQ_DIV_MAX;
 
         double set_freq = *f_acq;
-        if (set_freq <= 0)
-            set_freq = ref_freq;
+        if (set_freq <= 0) {
+            if (IS_E16_FREQ(hnd, ref_freq)) {
+                set_freq = E16_ADC_FREQ_DEFAULT;
+            } else {
+                set_freq = ref_freq;
+            }
+        }
+        if (IS_E16_FREQ(hnd, ref_freq)) {
+            div_max = E16_ADC_FREQ_DIV_MAX;
+            if (set_freq > E16_ADC_FREQ_MAX) {
+                set_freq = E16_ADC_FREQ_MAX;
+            }
+        }
+
         cur_adc_freq_div = (uint32_t)(ref_freq/set_freq+0.49);
         if (cur_adc_freq_div == 0)
             cur_adc_freq_div = 1;
-        if (cur_adc_freq_div > X502_ADC_FREQ_DIV_MAX)
-            cur_adc_freq_div = X502_ADC_FREQ_DIV_MAX;
+        if (cur_adc_freq_div > div_max)
+            cur_adc_freq_div = div_max;
         set_freq = ref_freq/cur_adc_freq_div;
 
         *f_acq = set_freq;
@@ -249,6 +290,11 @@ X502_EXPORT(int32_t) X502_CalcAdcFreq(double ref_freq, uint32_t lch_cnt, double
     return err;
 }
 
+X502_EXPORT(int32_t) X502_CalcAdcFreq(double ref_freq, uint32_t lch_cnt, double *f_acq,
+                                      double *f_frame, uint32_t *adc_freq_div, uint32_t *adc_frame_delay) {
+    return X502_CalcAdcFreq2(NULL, ref_freq, lch_cnt, f_acq, f_frame, adc_freq_div, adc_frame_delay);
+}
+
 X502_EXPORT(int32_t) X502_SetAdcFreq(t_x502_hnd hnd, double *f_acq, double *f_frame) {
     int32_t err = X502_CHECK_HND_OPENED(hnd);
     if (!err && (hnd->flags & PRIV_FLAGS_STREAM_RUN))
@@ -257,7 +303,7 @@ X502_EXPORT(int32_t) X502_SetAdcFreq(t_x502_hnd hnd, double *f_acq, double *f_fr
     if (!err) {
         double ref_freq;
         X502_GetRefFreqValue(hnd, &ref_freq);
-        err = X502_CalcAdcFreq(ref_freq, hnd->set.lch_cnt, f_acq, f_frame,
+        err = X502_CalcAdcFreq2(hnd, ref_freq, hnd->set.lch_cnt, f_acq, f_frame,
                                &hnd->set.adc_freq_div, &hnd->set.adc_frame_delay);
     }
     return err;
@@ -287,17 +333,31 @@ X502_EXPORT(int32_t) X502_GetAdcFreq(t_x502_hnd hnd, double *f_acq, double *f_fr
 }
 
 
-X502_EXPORT(int32_t) X502_CalcDinFreq(double ref_freq, double *f_din, uint32_t *result_freq_div) {
+X502_EXPORT(int32_t) X502_CalcDinFreq2(t_x502_hnd hnd, double ref_freq, double *f_din, uint32_t *result_freq_div) {
     int32_t err = f_din==NULL ? X502_ERR_INVALID_POINTER : X502_ERR_OK;
     if (err == X502_ERR_OK) {
+        uint32_t freq_div;
         double set_freq = *f_din;
-        if (set_freq<=0)
-            set_freq = ref_freq;
-        uint32_t freq_div = (uint32_t)(ref_freq/set_freq+0.49);
+        uint32_t div_max = X502_DIN_FREQ_DIV_MAX;
+
+        if (set_freq<=0) {
+            if (IS_E16_FREQ(hnd, ref_freq)) {
+                set_freq = E16_DIN_FREQ_DEFAULT;
+            } else {
+                set_freq = ref_freq;
+            }
+        }
+        if (IS_E16_FREQ(hnd, ref_freq)) {
+            div_max = E16_DIN_FREQ_DIV_MAX;
+            if (set_freq > E16_DIN_FREQ_MAX) {
+                set_freq = E16_DIN_FREQ_MAX;
+            }
+        }
+        freq_div = (uint32_t)(ref_freq/set_freq+0.49);
         if (freq_div == 0)
             freq_div = 1;
-        if (freq_div > X502_DIN_FREQ_DIV_MAX)
-            freq_div = X502_DIN_FREQ_DIV_MAX;
+        if (freq_div > div_max)
+            freq_div = div_max;
         set_freq = ref_freq/freq_div;
         *f_din = set_freq;
 
@@ -307,6 +367,10 @@ X502_EXPORT(int32_t) X502_CalcDinFreq(double ref_freq, double *f_din, uint32_t *
     return err;
 }
 
+X502_EXPORT(int32_t) X502_CalcDinFreq(double ref_freq, double *f_din, uint32_t *result_freq_div) {
+    return X502_CalcDinFreq2(NULL, ref_freq, f_din, result_freq_div);
+}
+
 X502_EXPORT(int32_t) X502_SetDinFreq(t_x502_hnd hnd, double *f_din) {
     int32_t err = X502_CHECK_HND_OPENED(hnd);
     if (!err && (hnd->flags & PRIV_FLAGS_STREAM_RUN))
@@ -314,24 +378,40 @@ X502_EXPORT(int32_t) X502_SetDinFreq(t_x502_hnd hnd, double *f_din) {
     if (!err) {
         double ref_freq;
         X502_GetRefFreqValue(hnd, &ref_freq);
-        err = X502_CalcDinFreq(ref_freq, f_din, &hnd->set.din_freq_div);
+        err = X502_CalcDinFreq2(hnd, ref_freq, f_din, &hnd->set.din_freq_div);
     }
     return err;
 }
 
 
-X502_EXPORT(int32_t) X502_CalcOutFreq(double ref_freq, double *f_dout, uint32_t *result_freq_div) {
+X502_EXPORT(int32_t) X502_CalcOutFreq2(t_x502_hnd hnd, double ref_freq, double *f_dout, uint32_t *result_freq_div) {
     int32_t err = f_dout==NULL ? X502_ERR_INVALID_POINTER : X502_ERR_OK;
     if (err == X502_ERR_OK) {
-        double set_freq = *f_dout;
-        if (set_freq<=0)
-            set_freq = ref_freq;
+        const uint32_t out_div_min = f_get_out_freq_div_min(ref_freq);
+        uint32_t out_freq_div;
+        uint32_t div_max = X502_OUT_FREQ_DIV_MAX;
 
-        uint32_t out_freq_div = (uint32_t)(ref_freq/set_freq+0.49);
-        if (out_freq_div < X502_OUT_FREQ_DIV_MIN)
-            out_freq_div = X502_OUT_FREQ_DIV_MIN;
-        if (out_freq_div > X502_OUT_FREQ_DIV_MAX)
-            out_freq_div = X502_OUT_FREQ_DIV_MAX;
+
+        double set_freq = *f_dout;
+        if (set_freq <= 0) {
+            if (IS_E16_FREQ(hnd, ref_freq)) {
+                set_freq = E16_OUT_FREQ_DEFAULT;
+            } else {
+                set_freq = ref_freq;
+            }
+        }
+        if (IS_E16_FREQ(hnd, ref_freq)) {
+            div_max = E16_OUT_FREQ_DIV_MAX;
+            if (set_freq > E16_OUT_FREQ_MAX) {
+                set_freq = E16_OUT_FREQ_MAX;
+            }
+        }
+
+        out_freq_div = (uint32_t)(ref_freq/set_freq+0.49);
+        if (out_freq_div < out_div_min)
+            out_freq_div = out_div_min;
+        if (out_freq_div > div_max)
+            out_freq_div = div_max;
 
         set_freq = ref_freq/out_freq_div;
         *f_dout = set_freq;
@@ -342,6 +422,10 @@ X502_EXPORT(int32_t) X502_CalcOutFreq(double ref_freq, double *f_dout, uint32_t
     return err;
 }
 
+X502_EXPORT(int32_t) X502_CalcOutFreq(double ref_freq, double *f_dout, uint32_t *result_freq_div) {
+    return X502_CalcOutFreq2(NULL, ref_freq, f_dout, result_freq_div);
+}
+
 X502_EXPORT(int32_t) X502_SetOutFreq(t_x502_hnd hnd, double *f_dout) {
     int32_t err = X502_CHECK_HND_OPENED(hnd);
     if (!err && (hnd->flags & PRIV_FLAGS_STREAM_RUN))
@@ -359,7 +443,7 @@ X502_EXPORT(int32_t) X502_SetOutFreq(t_x502_hnd hnd, double *f_dout) {
             hnd->set.out_freq_div = X502_OUT_FREQ_DIV_DEFAULT;
             *f_dout = ref_freq/hnd->set.out_freq_div;
         } else {
-            err = X502_CalcOutFreq(ref_freq, f_dout, &hnd->set.out_freq_div);
+            err = X502_CalcOutFreq2(hnd, ref_freq, f_dout, &hnd->set.out_freq_div);
         }
     }
     return err;
@@ -373,8 +457,14 @@ X502_EXPORT(int32_t) X502_SetRefFreq(t_x502_hnd hnd, uint32_t freq) {
     if (!err && (hnd->flags & PRIV_FLAGS_STREAM_RUN))
         err = X502_ERR_STREAM_IS_RUNNING;
 
-    if (!err)
-        hnd->set.ref_freq = freq;
+    if (!err) {
+        if (x502_is_E16(hnd)) {
+            // для E16 ref freq всегда 48 МГц
+            hnd->set.ref_freq = E16_REF_FREQ_48000KHZ;
+        } else {
+            hnd->set.ref_freq = freq;
+        }
+    }
     return err;
 }
 
@@ -389,6 +479,19 @@ X502_EXPORT(int32_t) X502_SetExtRefFreqValue(t_x502_hnd hnd, double freq) {
     return err;
 }
 
+X502_EXPORT(int32_t) X502_SetAdcSyncStartValue(t_x502_hnd hnd, uint32_t phy_ch, uint32_t mode, uint32_t range, double value) {
+    int32_t err = X502_CHECK_HND_OPENED(hnd);
+    if (!err && (hnd->flags & PRIV_FLAGS_STREAM_RUN))
+        err = X502_ERR_STREAM_IS_RUNNING;
+    if (!err) {
+        hnd->set.adc_sync_ch = phy_ch;
+        hnd->set.adc_sync_range = range;
+        hnd->set.adc_sync_mode = mode;
+        hnd->set.adc_sync_val = value;
+    }
+    return err;
+}
+
 X502_EXPORT(int32_t) X502_GetRefFreqValue(t_x502_hnd hnd, double *freq) {
     int32_t err = X502_CHECK_HND_OPENED(hnd);
 
@@ -410,8 +513,9 @@ X502_EXPORT(int32_t) X502_SetSyncMode(t_x502_hnd hnd, uint32_t sync_mode) {
     if (!err && (hnd->flags & PRIV_FLAGS_STREAM_RUN))
         err = X502_ERR_STREAM_IS_RUNNING;
 
-    if (!err)
+    if (!err) {
         err = CHECK_SYNC_MODE(sync_mode);
+    }
 
     if (!err) {
         hnd->set.sync_mode = sync_mode;
@@ -424,8 +528,19 @@ X502_EXPORT(int32_t) X502_SetSyncStartMode(t_x502_hnd hnd, uint32_t sync_start_m
     if (!err && (hnd->flags & PRIV_FLAGS_STREAM_RUN))
         err = X502_ERR_STREAM_IS_RUNNING;
 
-    if (!err)
-        err = CHECK_SYNC_MODE(sync_start_mode);
+    if (!err) {
+        if (sync_start_mode==E16_SYNC_ADC_EDGE_RISE ||
+            sync_start_mode==E16_SYNC_ADC_EDGE_FALL ||
+            sync_start_mode==E16_SYNC_ADC_ABOVE_LEVEL ||
+            sync_start_mode==E16_SYNC_ADC_BELOW_LEVEL) {
+            if (x502_is_E16(hnd) == false) {
+                // реализовано только в E16
+                return X502_ERR_INVALID_SYNC_MODE;
+            }
+        } else {
+            err = CHECK_SYNC_MODE(sync_start_mode);
+        }
+    }
 
     if (!err) {
         hnd->set.sync_start_mode = sync_start_mode;
@@ -434,6 +549,23 @@ X502_EXPORT(int32_t) X502_SetSyncStartMode(t_x502_hnd hnd, uint32_t sync_start_m
     return err;
 }
 
+uint32_t get_ltable_val(t_x502_hnd hnd, uint32_t phy_ch, uint32_t mode, uint32_t range, uint32_t avg) {
+    uint32_t wrd = ((phy_ch & 0xF) << 3) | (range & 0x7);
+
+    if (mode == X502_LCH_MODE_ZERO) {
+        wrd |= (3 << 7);
+    } else if (mode == X502_LCH_MODE_COMM) {
+        wrd |= (phy_ch & 0x10 ? 2 : 1) << 7;
+    }
+
+    if (avg) {
+        uint32_t avg_val = avg;
+        if (avg_val > hnd->set.adc_freq_div)
+            avg_val = hnd->set.adc_freq_div;
+        wrd |= ((avg_val-1) & 0x7F) << 9;
+    }
+    return wrd;
+}
 
 X502_EXPORT(int32_t) X502_Configure(t_x502_hnd hnd, uint32_t flags) {
     int32_t err = X502_CHECK_HND_OPENED(hnd);
@@ -445,6 +577,7 @@ X502_EXPORT(int32_t) X502_Configure(t_x502_hnd hnd, uint32_t flags) {
            Для внутренней может быть только одно из двух значений,
            для внешней главное, чтобы не принимало требуемой */
         if ((hnd->set.sync_mode==X502_SYNC_INTERNAL)
+                && (hnd->set.ref_freq!=E16_REF_FREQ_48000KHZ)
                 && (hnd->set.ref_freq!=X502_REF_FREQ_2000KHZ)
                 &&(hnd->set.ref_freq!=X502_REF_FREQ_1500KHZ)) {
             err = X502_ERR_INVALID_REF_FREQ;
@@ -458,20 +591,7 @@ X502_EXPORT(int32_t) X502_Configure(t_x502_hnd hnd, uint32_t flags) {
             uint32_t ch;
             /* записываем логическую таблицу */
             for (ch=0; (ch < hnd->set.lch_cnt) && !err; ch++) {
-                uint32_t wrd = ((hnd->set.lch[ch].ch & 0xF) << 3) | (hnd->set.lch[ch].range & 0x7);
-
-                if (hnd->set.lch[ch].mode == X502_LCH_MODE_ZERO) {
-                    wrd |= (3 << 7);
-                } else if (hnd->set.lch[ch].mode == X502_LCH_MODE_COMM) {
-                    wrd |= (hnd->set.lch[ch].ch & 0x10 ? 2 : 1) << 7;
-                }
-
-                if (hnd->set.lch[ch].avg) {
-                    uint32_t avg = hnd->set.lch[ch].avg;
-                    if (avg > hnd->set.adc_freq_div)
-                        avg = hnd->set.adc_freq_div;
-                    wrd |= ((avg-1) & 0x7F) << 9;
-                }
+                uint32_t wrd = get_ltable_val(hnd, hnd->set.lch[ch].ch, hnd->set.lch[ch].mode, hnd->set.lch[ch].range, hnd->set.lch[ch].avg);
 
                 err = hnd->iface_hnd->fpga_reg_write(hnd, (X502_REGS_IOHARD_LTABLE +
                                                       hnd->set.lch_cnt - 1 - ch) & 0xFFFF, wrd);
@@ -498,12 +618,27 @@ X502_EXPORT(int32_t) X502_Configure(t_x502_hnd hnd, uint32_t flags) {
             }
 
             if (!err) {
+                // для варианта E16 ref_freq будет всегда 48000000
                 err = hnd->iface_hnd->fpga_reg_write(hnd, X502_REGS_IOHARD_IO_MODE,
                                                  (SYNC_FREQ_CODE(hnd->set.sync_mode) & 0x7)
-                                              | ((hnd->set.sync_start_mode&0x7)<<3)
-                                              | ((hnd->set.ref_freq==X502_REF_FREQ_2000KHZ ?
-                                                      0 : 2) << 7)
-                                              | (((hnd->set.out_freq_div-1)&0x3FF)<<9));
+                                              | ((hnd->set.sync_start_mode&0xF)<<3)
+                                              | ((hnd->set.ref_freq==X502_REF_FREQ_2000KHZ ? X502_MODE_REF_FREQ_2000 :
+                    (hnd->set.ref_freq==X502_REF_FREQ_1500KHZ ? X502_MODE_REF_FREQ_1500: E16_MODE_REF_FREQ_48000)) << 7)
+                                              | (((hnd->set.out_freq_div-1) & (x502_is_E16(hnd) ? 0x7FFF : 0x3FF))<<9));
+            }
+
+            if (x502_is_E16(hnd)) {
+                // старт по аналоговому сигналу реализован только в E16
+                if (!err) {
+                    uint32_t wrd = get_ltable_val(hnd, hnd->set.adc_sync_ch, hnd->set.adc_sync_mode, hnd->set.adc_sync_range, 0);
+
+                    err = hnd->iface_hnd->fpga_reg_write(hnd, X502_REGS_IOARITH_N_CHAN_SYN, wrd);
+                }
+                if (!err) {
+                    uint32_t threshold;
+                    threshold = hnd->set.adc_sync_val / hnd->set.f_scales[hnd->set.adc_sync_range] * E16_ADC_SCALE_CODE_MAX;
+                    err = hnd->iface_hnd->fpga_reg_write(hnd, X502_REGS_IOARITH_THRESHOLD, threshold);
+                }
             }
 
             if (!err) {
@@ -576,8 +711,14 @@ X502_EXPORT(int32_t) X502_SetAdcCoef(t_x502_hnd hnd, uint32_t range, double k, d
         err = X502_ERR_STREAM_IS_RUNNING;
 
     if (!err) {
-        if (range >= X502_ADC_RANGE_CNT) {
-            err = X502_ERR_INVALID_LCH_RANGE;
+        if (x502_is_E16(hnd)) {
+            if (range >= E16_ADC_RANGE_CNT) {
+                err = X502_ERR_INVALID_LCH_RANGE;
+            }
+        } else {
+            if (range >= X502_ADC_RANGE_CNT) {
+                err = X502_ERR_INVALID_LCH_RANGE;
+            }
         }
     }
 
