@@ -134,11 +134,7 @@ X502_EXPORT(int32_t) X502_StreamsEnable(t_x502_hnd hnd, uint32_t streams) {
         } else {
             if (hnd->flags & PRIV_FLAGS_STREAM_RUN) {
                 uint32_t old_streams = hnd->streams;
-                /* Запись в регистры плиса имеет смысл делать только перез запуском
-                   или во время запуска синхронного сбора, т.к. пока сбор не запущен
-                   - не вляют. но в старых версиях ПЛИС могли приводит к сдвигу по
-                   времени первого отсчета АЦП */
-                err = f_set_streams(hnd, hnd->streams | streams);
+                hnd->streams |= streams;
 
                 /* если не было разрешено потока на ввод до этого,
                    а при вызове стал разрешен => инициализируем его */
@@ -151,6 +147,15 @@ X502_EXPORT(int32_t) X502_StreamsEnable(t_x502_hnd hnd, uint32_t streams) {
                                   (streams & X502_STREAM_ALL_IN))) {
                     err =f_stream_in_cfg(hnd);
                 }
+
+
+                /* Запись в регистры плиса имеет смысл делать только перед запуском
+                   или во время запуска синхронного сбора, т.к. пока сбор не запущен
+                   - не вляют. но в старых версиях ПЛИС могли приводит к сдвигу по
+                   времени первого отсчета АЦП.
+                   Разрешение потока в ПЛИС следует делать уже после stream_start,
+                   чтобы ПК был готов принимать данные */
+                err = f_set_streams(hnd, hnd->streams | streams);
             } else {
                 hnd->streams |= streams;
             }
@@ -241,7 +246,7 @@ X502_EXPORT(int32_t) X502_StreamsStart(t_x502_hnd hnd) {
              * предзагружены в буфер платы */
             if ((hnd->streams & X502_STREAM_ALL_OUT) &&
                     (hnd->flags & (PRIV_FLAGS_PRELOAD_DONE | PRIV_FLAGS_CYCLE_MODE))) {
-                err = hnd->iface_hnd->fpga_reg_write(hnd, X502_REGS_IOHARD_OUTSWAP_BFCTL, 1);                
+                err = hnd->iface_hnd->fpga_reg_write(hnd, X502_REGS_IOHARD_OUTSWAP_BFCTL, 1);
             }
 
 
@@ -258,7 +263,6 @@ X502_EXPORT(int32_t) X502_StreamsStart(t_x502_hnd hnd) {
                 err = hnd->iface_hnd->fpga_reg_write(hnd, X502_REGS_IOHARD_PRELOAD_ADC, 1);
             if (err == X502_ERR_OK)
                 err = hnd->iface_hnd->fpga_reg_write(hnd, X502_REGS_IOHARD_PRELOAD_ADC, 1);
-
             SLEEP_MS(20);
         }
 
@@ -309,7 +313,7 @@ X502_EXPORT(int32_t) X502_StreamsStop(t_x502_hnd hnd) {
 
         if (hnd->mode==X502_MODE_FPGA) {
             err = hnd->iface_hnd->fpga_reg_write(hnd, X502_REGS_IOHARD_GO_SYNC_IO, 0);
-        } else if (hnd->mode == X502_MODE_DSP) {           
+        } else if (hnd->mode == X502_MODE_DSP) {
             err = X502_BfExecCmd(hnd, L502_BF_CMD_CODE_STREAM_STOP, 0,
                               NULL, 0, NULL, 0, X502_BF_CMD_DEFAULT_TOUT, NULL);
         }
@@ -487,9 +491,16 @@ static int32_t f_cycle_setup_wait(t_x502_hnd hnd, uint32_t tout) {
     int32_t err = X502_ERR_OK;
     t_ltimer tmr;
     uint32_t done = 0;
+    int first = 1;
     ltimer_set(&tmr, LTIMER_MS_TO_CLOCK_TICKS(tout));
 
     do {
+        if (!first) {
+            SLEEP_MS(5);
+        } else {
+            first = 0;
+        }
+
         err = X502_OutCycleCheckSetupDone(hnd, &done);
     } while (!done && !ltimer_expired(&tmr) && (err == X502_ERR_OK));
 
